@@ -12,7 +12,10 @@ lazy_static! {
     // For more information, see:
     // https://github.com/BurntSushi/ripgrep/discussions/1623#discussioncomment-28827
     static ref SUBJECT_STARTS_WITH_EMOJI: Regex = Regex::new(r"^[\p{Emoji}--\p{Ascii}]").unwrap();
-    static ref SUBJECT_WITH_TICKET: Regex = Regex::new(r"[A-Z]+-\d+").unwrap();
+    // Jira project keys are at least 2 uppercase characters long.
+    // AB-123
+    // JIRA-123
+    static ref SUBJECT_WITH_TICKET: Regex = Regex::new(r"[A-Z]{2,}-\d+").unwrap();
     // Match all GitHub and GitLab keywords
     static ref SUBJECT_WITH_FIX_TICKET: Regex =
         Regex::new(r"([fF]ix(es|ed|ing)?|[cC]los(e|es|ed|ing)|[rR]esolv(e|es|ed|ing)|[iI]mplement(s|ed|ing)?):? ([^\s]*[\w\-_/]+)?[#!]{1}\d+").unwrap();
@@ -22,6 +25,12 @@ lazy_static! {
         tempregex.multi_line(false);
         tempregex.build().unwrap()
     };
+    static ref SUBJECT_WITH_BUILD_TAGS: Regex = {
+        let mut tempregex = RegexBuilder::new(r"(\[(skip [\w\s_-]+|[\w\s_-]+ skip|no ci)\]|\*\*\*NO_CI\*\*\*)");
+        tempregex.case_insensitive(true);
+        tempregex.multi_line(false);
+        tempregex.build().unwrap()
+    };
 
     static ref URL_REGEX: Regex = Regex::new(r"https?://\w+").unwrap();
     static ref CODE_BLOCK_LINE_WITH_LANGUAGE: Regex = Regex::new(r"^\s*```\s*([\w]+)?$").unwrap();
@@ -69,34 +78,6 @@ lazy_static! {
         "tested",
         "testing",
     ];
-    static ref BUILD_TAGS: Vec<&'static str> = vec![
-        // General
-        "[ci skip]",
-        "[skip ci]",
-        "[no ci]",
-        // AppVeyor
-        "[skip appveyor]",
-        // Azure
-        "[azurepipelines skip]",
-        "[skip azurepipelines]",
-        "[azpipelines skip]",
-        "[skip azpipelines]",
-        "[azp skip]",
-        "[skip azp]",
-        "***NO_CI***",
-        // GitHub Actions
-        "[actions skip]",
-        "[skip actions]",
-        // Travis
-        "[travis skip]",
-        "[skip travis]",
-        "[travis ci skip]",
-        "[skip travis ci]",
-        "[travis-ci skip]",
-        "[skip travis-ci]",
-        "[travisci skip]",
-        "[skip travisci]",
-    ];
 }
 
 #[derive(Debug)]
@@ -106,6 +87,7 @@ pub struct Commit {
     pub email: Option<String>,
     pub subject: String,
     pub message: String,
+    pub has_changes: bool,
     pub violations: Vec<Violation>,
     pub ignored: bool,
     pub ignored_rules: Vec<Rule>,
@@ -117,6 +99,7 @@ impl Commit {
         email: Option<String>,
         subject: String,
         message: String,
+        has_changes: bool,
     ) -> Self {
         // Get first 7 characters of the commit SHA to get the short SHA.
         let short_sha = match &long_sha {
@@ -136,6 +119,7 @@ impl Commit {
             email,
             subject: subject.trim_end().to_string(),
             message,
+            has_changes,
             ignored: false,
             ignored_rules,
             violations: Vec::<Violation>::new(),
@@ -167,18 +151,19 @@ impl Commit {
     pub fn validate(&mut self) {
         self.validate_merge_commit();
         self.validate_needs_rebase();
+        self.validate_subject_cliches();
         self.validate_subject_line_length();
         self.validate_subject_mood();
         self.validate_subject_whitespace();
+        self.validate_subject_prefix();
         self.validate_subject_capitalization();
         self.validate_subject_punctuation();
         self.validate_subject_ticket_numbers();
-        self.validate_subject_prefix();
         self.validate_subject_build_tags();
-        self.validate_subject_cliches();
-        self.validate_message_second_line_empty();
+        self.validate_message_empty_first_line();
         self.validate_message_presence();
         self.validate_message_line_length();
+        self.validate_changes();
     }
 
     // Note: Some merge commits are ignored in git.rs and won't be validated here, because they are
@@ -217,11 +202,19 @@ impl Commit {
     }
 
     fn validate_subject_line_length(&mut self) {
-        if self.rule_ignored(Rule::SubjectLength) {
+        if self.rule_ignored(Rule::SubjectLength) || self.has_violation(Rule::SubjectCliche) {
             return;
         }
 
         let length = self.subject.chars().count();
+        if length == 0 {
+            self.add_violation(
+                Rule::SubjectLength,
+                "The subject is not present. Please add a subject to describe the change."
+                    .to_string(),
+            );
+            return;
+        }
         if length > 50 {
             self.add_violation(
                 Rule::SubjectLength,
@@ -267,6 +260,9 @@ impl Commit {
         if self.rule_ignored(Rule::SubjectWhitespace) {
             return;
         }
+        if self.subject.chars().count() == 0 && self.has_violation(Rule::SubjectLength) {
+            return;
+        }
 
         match self.subject.chars().next() {
             Some(character) => {
@@ -284,7 +280,13 @@ impl Commit {
     }
 
     fn validate_subject_capitalization(&mut self) {
-        if self.rule_ignored(Rule::SubjectCapitalization) {
+        if self.rule_ignored(Rule::SubjectCapitalization)
+            || self.has_violation(Rule::NeedsRebase)
+            || self.has_violation(Rule::SubjectPrefix)
+        {
+            return;
+        }
+        if self.subject.chars().count() == 0 && self.has_violation(Rule::SubjectLength) {
             return;
         }
 
@@ -307,6 +309,9 @@ impl Commit {
         if self.rule_ignored(Rule::SubjectPunctuation) {
             return;
         }
+        if self.subject.chars().count() == 0 && self.has_violation(Rule::SubjectLength) {
+            return;
+        }
 
         if SUBJECT_STARTS_WITH_EMOJI.is_match(&self.subject) {
             self.add_violation(
@@ -397,12 +402,13 @@ impl Commit {
         }
 
         let subject = &self.subject.to_string();
-        for tag in BUILD_TAGS.iter() {
-            if subject.contains(tag) {
-                self.add_violation(
+        if let Some(captures) = SUBJECT_WITH_BUILD_TAGS.captures(subject) {
+            match captures.get(1) {
+                Some(tag) => self.add_violation(
                     Rule::SubjectBuildTag,
-                    format!("Move the build tag `{}` to the message body.", tag),
-                )
+                    format!("Move the build tag `{}` to the message body.", tag.as_str()),
+                ),
+                None => error!("SubjectBuildTag: Unable to fetch build tag from subject."),
             }
         }
     }
@@ -422,7 +428,7 @@ impl Commit {
         }
     }
 
-    fn validate_message_second_line_empty(&mut self) {
+    fn validate_message_empty_first_line(&mut self) {
         if self.rule_ignored(Rule::MessageEmptyFirstLine) {
             return;
         }
@@ -438,7 +444,7 @@ impl Commit {
     }
 
     fn validate_message_presence(&mut self) {
-        if self.rule_ignored(Rule::MessagePresence) {
+        if self.rule_ignored(Rule::MessagePresence) || self.has_violation(Rule::NeedsRebase) {
             return;
         }
 
@@ -508,9 +514,29 @@ impl Commit {
         }
     }
 
+    fn validate_changes(&mut self) {
+        if self.rule_ignored(Rule::DiffPresence) {
+            return;
+        }
+
+        if !self.has_changes {
+            self.add_violation(
+                Rule::DiffPresence,
+                "The commit has no file changes. Add changes to the commit or remove the commit."
+                    .to_string(),
+            )
+        }
+    }
+
     fn add_violation(&mut self, rule: Rule, message: String) {
         self.violations.push(Violation { rule, message })
     }
+
+    fn has_violation(&self, rule: Rule) -> bool {
+        self.violations
+            .iter()
+            .any(|violation| violation.rule == rule)
+    }
 }
 
 #[derive(PartialEq)]
@@ -522,10 +548,16 @@ enum CodeBlockStyle {
 
 #[cfg(test)]
 mod tests {
-    use super::{Commit, Rule, Violation, BUILD_TAGS, MOOD_WORDS};
+    use super::{Commit, Rule, Violation, MOOD_WORDS};
 
     fn commit_with_sha(sha: Option<String>, subject: String, message: String) -> Commit {
-        Commit::new(sha, Some("test@example.com".to_string()), subject, message)
+        Commit::new(
+            sha,
+            Some("test@example.com".to_string()),
+            subject,
+            message,
+            true,
+        )
     }
 
     fn commit(subject: String, message: String) -> Commit {
@@ -536,6 +568,16 @@ mod tests {
         )
     }
 
+    fn commit_without_file_changes(message: String) -> Commit {
+        Commit::new(
+            Some("aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa".to_string()),
+            Some("test@example.com".to_string()),
+            "Some subject".to_string(),
+            message,
+            false,
+        )
+    }
+
     fn validated_commit(subject: String, message: String) -> Commit {
         let mut commit = commit(subject, message);
         commit.validate();
@@ -679,6 +721,9 @@ mod tests {
             "lintje:disable SubjectLength".to_string(),
         );
         assert_commit_valid_for(ignore_commit, &Rule::SubjectLength);
+
+        // Already a SubjectCliche violation, so it's skipped.
+        assert_commit_subject_as_valid("wip", &Rule::SubjectLength);
     }
 
     #[test]
@@ -715,6 +760,9 @@ mod tests {
         let invalid_subjects = vec![" Fix test", "\tFix test", "\x20Fix test"];
         assert_commit_subjects_as_invalid(invalid_subjects, &Rule::SubjectWhitespace);
 
+        assert_commit_subject_as_invalid("", &Rule::SubjectLength);
+        assert_commit_subject_as_valid("", &Rule::SubjectWhitespace);
+
         let ignore_commit = validated_commit(
             " Fix test".to_string(),
             "lintje:disable SubjectWhitespace".to_string(),
@@ -730,11 +778,26 @@ mod tests {
         let invalid_subjects = vec!["fix test"];
         assert_commit_subjects_as_invalid(invalid_subjects, &Rule::SubjectCapitalization);
 
+        assert_commit_subject_as_invalid("", &Rule::SubjectLength);
+        assert_commit_subject_as_valid("", &Rule::SubjectCapitalization);
+
         let ignore_commit = validated_commit(
             "fix test".to_string(),
             "lintje:disable SubjectCapitalization".to_string(),
         );
         assert_commit_valid_for(ignore_commit, &Rule::SubjectCapitalization);
+
+        // Already a NeedsRebase violation, so it's skipped.
+        let rebase_commit = validated_commit("fixup! foo".to_string(), "".to_string());
+        assert_commit_valid_for(rebase_commit, &Rule::SubjectCapitalization);
+        let rebase_commit = validated_commit("fixup! foo".to_string(), "".to_string());
+        assert_commit_invalid_for(rebase_commit, &Rule::NeedsRebase);
+
+        // Already a SubjectPrefix violation, so it's skipped.
+        let prefix_commit = validated_commit("chore: foo".to_string(), "".to_string());
+        assert_commit_valid_for(prefix_commit, &Rule::SubjectCapitalization);
+        let prefix_commit = validated_commit("chore: foo".to_string(), "".to_string());
+        assert_commit_invalid_for(prefix_commit, &Rule::SubjectPrefix);
     }
 
     #[test]
@@ -785,6 +848,9 @@ mod tests {
         ];
         assert_commit_subjects_as_invalid(invalid_subjects, &Rule::SubjectPunctuation);
 
+        assert_commit_subject_as_invalid("", &Rule::SubjectLength);
+        assert_commit_subject_as_valid("", &Rule::SubjectPunctuation);
+
         let ignore_commit = validated_commit(
             "Fix test.".to_string(),
             "lintje:disable SubjectPunctuation".to_string(),
@@ -802,10 +868,24 @@ mod tests {
             "Fix !",
             "Fix !!123",
             "Fix !a123",
+            "Change A-1 config",
+            "Change A-12 config",
         ];
         assert_commit_subjects_as_valid(valid_ticket_subjects, &Rule::SubjectTicketNumber);
 
-        let invalid_ticket_subjects = vec!["JIRA-1234", "Fix JIRA-1234 lorem"];
+        let invalid_ticket_subjects = vec![
+            "JI-1",
+            "JI-12",
+            "JI-1234567890",
+            "JIR-1",
+            "JIR-12",
+            "JIR-1234567890",
+            "JIRA-12",
+            "JIRA-123",
+            "JIRA-1234",
+            "JIRA-1234567890",
+            "Fix JIRA-1234 lorem",
+        ];
         assert_commit_subjects_as_invalid(invalid_ticket_subjects, &Rule::SubjectTicketNumber);
 
         let invalid_subjects = vec![
@@ -909,8 +989,40 @@ mod tests {
         let subjects = vec!["Add exception for no ci build tag"];
         assert_commit_subjects_as_valid(subjects, &Rule::SubjectBuildTag);
 
+        let build_tags = vec![
+            // General
+            "[ci skip]",
+            "[skip ci]",
+            "[no ci]",
+            // AppVeyor
+            "[skip appveyor]",
+            // Azure
+            "[azurepipelines skip]",
+            "[skip azurepipelines]",
+            "[azpipelines skip]",
+            "[skip azpipelines]",
+            "[azp skip]",
+            "[skip azp]",
+            "***NO_CI***",
+            // GitHub Actions
+            "[actions skip]",
+            "[skip actions]",
+            // Travis
+            "[travis skip]",
+            "[skip travis]",
+            "[travis ci skip]",
+            "[skip travis ci]",
+            "[travis-ci skip]",
+            "[skip travis-ci]",
+            "[travisci skip]",
+            "[skip travisci]",
+            // Other custom tags that match the format
+            "[skip me]",
+            "[skip changeset]",
+            "[skip review]",
+        ];
         let mut invalid_subjects = vec![];
-        for tag in BUILD_TAGS.iter() {
+        for tag in build_tags.iter() {
             invalid_subjects.push(format!("Update README {}", tag))
         }
         assert_commit_subjects_as_invalid(invalid_subjects, &Rule::SubjectBuildTag);
@@ -1009,6 +1121,12 @@ mod tests {
             "lintje:disable MessagePresence".to_string(),
         );
         assert_commit_valid_for(ignore_commit, &Rule::MessagePresence);
+
+        // Already a NeedsRebase violation, so it's skipped.
+        let rebase_commit = validated_commit("fixup! foo".to_string(), "".to_string());
+        assert_commit_valid_for(rebase_commit, &Rule::MessagePresence);
+        let rebase_commit = validated_commit("fixup! foo".to_string(), "".to_string());
+        assert_commit_invalid_for(rebase_commit, &Rule::NeedsRebase);
     }
 
     #[test]
@@ -1171,4 +1289,20 @@ mod tests {
             &Rule::MessageLineLength,
         );
     }
+
+    #[test]
+    fn test_validate_changes_presense() {
+        let with_changes = validated_commit("Subject".to_string(), "\nSome message.".to_string());
+        assert_commit_valid_for(with_changes, &Rule::DiffPresence);
+
+        let mut without_changes = commit_without_file_changes("\nSome message.".to_string());
+        without_changes.validate();
+        assert_commit_invalid_for(without_changes, &Rule::DiffPresence);
+
+        let mut ignore_commit = commit_without_file_changes(
+            "\nSome message.\nlintje:disable: DiffPresence".to_string(),
+        );
+        ignore_commit.validate();
+        assert_commit_invalid_for(ignore_commit, &Rule::DiffPresence);
+    }
 }
