@@ -21,6 +21,7 @@ mod rule;
 mod utils;
 
 use branch::Branch;
+use command::run_command;
 use commit::Commit;
 use git::{fetch_and_parse_branch, fetch_and_parse_commits, parse_commit_hook_format};
 use logger::Logger;
@@ -109,8 +110,25 @@ fn lint_commit_hook(filename: &Path) -> Result<Vec<Commit>, String> {
                     ));
                 }
             };
-            let commit =
-                parse_commit_hook_format(&contents, git::cleanup_mode(), git::comment_char());
+
+            // Run the diff command to fetch the current staged changes and determine if the commit is
+            // empty or not. The contents of the commit message file is too unreliable as it depends on
+            // user config and how the user called the `git commit` command.
+            let mut has_changes = true;
+            match run_command("git", &["diff", "--cached", "--shortstat"]) {
+                Ok(stdout) => {
+                    if stdout.is_empty() {
+                        has_changes = false;
+                    }
+                }
+                Err(e) => error!("Unable to determine commit changes.\nError: {}", e.message),
+            }
+            let commit = parse_commit_hook_format(
+                &contents,
+                git::cleanup_mode(),
+                git::comment_char(),
+                has_changes,
+            );
             vec![commit]
         }
         Err(e) => {
@@ -249,7 +267,7 @@ mod tests {
         Path::new(TEST_DIR).join(name)
     }
 
-    fn create_test_repo(dir: &PathBuf, commits: &[(&str, &str)]) {
+    fn create_test_repo(dir: &PathBuf) {
         if Path::new(&dir).exists() {
             fs::remove_dir_all(&dir).expect("Could not remove test repo dir");
         }
@@ -272,9 +290,6 @@ mod tests {
             )
         }
         create_commit(dir, "Initial commit", "");
-        for (subject, message) in commits {
-            create_commit(dir, subject, message)
-        }
     }
 
     fn checkout_branch(dir: &PathBuf, name: &str) {
@@ -328,6 +343,44 @@ mod tests {
         }
     }
 
+    fn create_commit_with_file(dir: &PathBuf, subject: &str, message: &str, filename: &str) {
+        create_file(&dir.join(&filename));
+        stage_files(&dir);
+        create_commit(&dir, &subject, &message)
+    }
+
+    fn create_file(file_path: &PathBuf) {
+        let mut file = match File::create(&file_path) {
+            Ok(file) => file,
+            Err(e) => panic!("Could not create file: {:?}: {}", file_path, e),
+        };
+        // Write a slice of bytes to the file
+        match file.write_all(b"I am a test file!") {
+            Ok(_) => (),
+            Err(e) => panic!("Could not write to file: {:?}: {}", file_path, e),
+        }
+    }
+
+    fn stage_files(dir: &PathBuf) {
+        let output = Command::new("git")
+            .args(["add", "."])
+            .current_dir(dir)
+            .stdin(Stdio::null())
+            .output()
+            .unwrap_or_else(|e| panic!("Failed to add files to commit: {:?}", e));
+        if !output.status.success() {
+            panic!(
+                "Failed to add files to commit!\nExit code: {}\nSDTOUT: {}\nSTDERR: {}",
+                output
+                    .status
+                    .code()
+                    .expect("Could not fetch status code of git add"),
+                String::from_utf8(output.stdout).unwrap(),
+                String::from_utf8(output.stderr).unwrap()
+            )
+        }
+    }
+
     fn configure_git_cleanup_mode(dir: &PathBuf, mode: &str) {
         let output = Command::new("git")
             .args(&["config", "commit.cleanup", mode])
@@ -382,7 +435,8 @@ mod tests {
     fn test_commit_by_sha() {
         compile_bin();
         let dir = test_dir("commit_by_sha");
-        create_test_repo(&dir, &[("Test commit", "")]);
+        create_test_repo(&dir);
+        create_commit_with_file(&dir, "Test commit", "", "file");
         let output = Command::new("git")
             .args(&["log", "--pretty=%H", "-n 1"])
             .current_dir(&dir)
@@ -405,10 +459,8 @@ mod tests {
     fn test_single_commit_valid() {
         compile_bin();
         let dir = test_dir("single_commit_valid");
-        create_test_repo(
-            &dir,
-            &[("Test commit", "I am a test commit, short but valid.")],
-        );
+        create_test_repo(&dir);
+        create_commit_with_file(&dir, "Test commit", "I am a test commit", "file");
 
         let mut cmd = assert_cmd::Command::cargo_bin("lintje").unwrap();
         let assert = cmd.current_dir(dir).assert().success();
@@ -421,15 +473,16 @@ mod tests {
     fn test_single_commit_invalid() {
         compile_bin();
         let dir = test_dir("single_commit_invalid");
-        create_test_repo(&dir, &[("added some code", ""), ("Fixing tests", "")]);
+        create_test_repo(&dir);
+        create_commit_with_file(&dir, "Fixing tests", "", "file");
 
         let mut cmd = assert_cmd::Command::cargo_bin("lintje").unwrap();
         let assert = cmd.current_dir(dir).assert().failure().code(1);
         assert
             .stdout(predicate::str::contains(
                 "Fixing tests\n\
-                \x20\x20SubjectMood: Use the imperative mood for the commit subject.\n\
                 \x20\x20SubjectCliche: Reword the subject to describe the change in more detail.\n\
+                \x20\x20SubjectMood: Use the imperative mood for the commit subject.\n\
                 \x20\x20MessagePresence: Add a message body to provide more context about the change and why it was made.",
             ))
             .stdout(predicate::str::contains(
@@ -441,10 +494,8 @@ mod tests {
     fn test_single_commit_invalid_one_violation() {
         compile_bin();
         let dir = test_dir("single_commit_invalid_one_violation");
-        create_test_repo(
-            &dir,
-            &[("added some code", ""), ("Valid commit subject", "")],
-        );
+        create_test_repo(&dir);
+        create_commit_with_file(&dir, "Valid commit subject", "", "file");
 
         let mut cmd = assert_cmd::Command::cargo_bin("lintje").unwrap();
         let assert = cmd.current_dir(dir).assert().failure().code(1);
@@ -458,16 +509,36 @@ mod tests {
             ));
     }
 
+    #[test]
+    fn test_single_commit_invalid_without_file_changes() {
+        compile_bin();
+        let dir = test_dir("single_commit_invalid_without_file_changes");
+        create_test_repo(&dir);
+        create_commit(&dir, "Valid commit subject", "");
+
+        let mut cmd = assert_cmd::Command::cargo_bin("lintje").unwrap();
+        let assert = cmd.current_dir(dir).assert().failure().code(1);
+        assert
+            .stdout(predicate::str::contains(
+                "Valid commit subject\n\
+                \x20\x20MessagePresence: Add a message body to provide more context about the change and why it was made.\n\
+                \x20\x20DiffPresence: The commit has no file changes. Add changes to the commit or remove the commit.",
+            ))
+            .stdout(predicate::str::contains(
+                "1 commit and branch inspected, 2 violations detected\n",
+            ));
+    }
+
     #[test]
     fn test_single_commit_ignored() {
         compile_bin();
         let dir = test_dir("single_commit_ignored");
-        create_test_repo(
+        create_test_repo(&dir);
+        create_commit_with_file(
             &dir,
-            &[
-                ("added some code", ""),
-                ("Merge pull request #123 from tombruijn/repo", ""),
-            ],
+            "Merge pull request #123 from tombruijn/repo",
+            "",
+            "file",
         );
 
         let mut cmd = assert_cmd::Command::cargo_bin("lintje").unwrap();
@@ -481,13 +552,8 @@ mod tests {
     fn test_single_commit_with_debug() {
         compile_bin();
         let dir = test_dir("single_commit_with_debug");
-        create_test_repo(
-            &dir,
-            &[
-                ("added some code", ""),
-                ("Valid commit subject", "Valid message body"),
-            ],
-        );
+        create_test_repo(&dir);
+        create_commit_with_file(&dir, "Valid commit subject", "Valid message body", "file");
 
         let mut cmd = assert_cmd::Command::cargo_bin("lintje").unwrap();
         let assert = cmd.current_dir(dir).args(["--debug"]).assert().success();
@@ -500,13 +566,9 @@ mod tests {
     fn test_multiple_commit_invalid() {
         compile_bin();
         let dir = test_dir("multiple_commits_invalid");
-        create_test_repo(
-            &dir,
-            &[
-                ("added some code", "This is a message."),
-                ("Fixing tests", ""),
-            ],
-        );
+        create_test_repo(&dir);
+        create_commit_with_file(&dir, "added some code", "This is a message.", "file1");
+        create_commit_with_file(&dir, "Fixing tests", "", "file2");
 
         let mut cmd = assert_cmd::Command::cargo_bin("lintje").unwrap();
         let assert = cmd
@@ -523,8 +585,8 @@ mod tests {
             ))
             .stdout(predicate::str::contains(
                 "Fixing tests\n\
-                \x20\x20SubjectMood: Use the imperative mood for the commit subject.\n\
                 \x20\x20SubjectCliche: Reword the subject to describe the change in more detail.\n\
+                \x20\x20SubjectMood: Use the imperative mood for the commit subject.\n\
                 \x20\x20MessagePresence: Add a message body to provide more context about the change and why it was made.",
             ))
             .stdout(predicate::str::contains(
@@ -536,7 +598,7 @@ mod tests {
     fn test_lint_hook() {
         compile_bin();
         let dir = test_dir("commit_file_option");
-        create_test_repo(&dir, &[]);
+        create_test_repo(&dir);
         let filename = "commit_message_file";
         let commit_file = dir.join(filename);
         let mut file = File::create(&commit_file).unwrap();
@@ -550,10 +612,39 @@ mod tests {
             .assert()
             .failure()
             .code(1);
+        assert
+            .stdout(predicate::str::contains(
+                    "added some code\n\
+                    \x20\x20SubjectMood: Use the imperative mood for the commit subject.\n\
+                    \x20\x20SubjectCapitalization: Start the commit subject with a capital letter.\n\
+                    \x20\x20DiffPresence: The commit has no file changes. Add changes to the commit or remove the commit.",
+            ))
+            .stdout(predicate::str::contains(
+                    "1 commit and branch inspected, 3 violations detected\n",
+            ));
+    }
+
+    #[test]
+    fn test_file_option_with_file_changes() {
+        compile_bin();
+        let dir = test_dir("commit_file_option_with_file_changes");
+        create_test_repo(&dir);
+        create_file(&dir.join("file name"));
+        stage_files(&dir);
+        let filename = "commit_message_file";
+        let commit_file = dir.join(filename);
+        let mut file = File::create(&commit_file).unwrap();
+        file.write_all(b"Valid subject\n\nValid message body.")
+            .unwrap();
+
+        let mut cmd = assert_cmd::Command::cargo_bin("lintje").unwrap();
+        let assert = cmd
+            .arg(&format!("--hook-message-file={}", filename))
+            .current_dir(dir)
+            .assert()
+            .success();
         assert.stdout(predicate::str::contains(
-            "added some code\n\
-             \x20\x20SubjectMood: Use the imperative mood for the commit subject.\n\
-             \x20\x20SubjectCapitalization: Start the commit subject with a capital letter.",
+            "1 commit and branch inspected, 0 violations detected\n",
         ));
     }
 
@@ -561,7 +652,7 @@ mod tests {
     fn test_file_option_with_scissors_cleanup() {
         compile_bin();
         let dir = test_dir("commit_file_option_with_scissors_cleanup_default_comment_char");
-        create_test_repo(&dir, &[]);
+        create_test_repo(&dir);
         configure_git_cleanup_mode(&dir, "scissors");
         let filename = "commit_message_file";
         let commit_file = dir.join(filename);
@@ -588,7 +679,7 @@ mod tests {
     fn test_file_option_with_scissors_cleanup_custom_comment_char() {
         compile_bin();
         let dir = test_dir("commit_file_option_with_scissors_cleanup_custom_comment_char");
-        create_test_repo(&dir, &[]);
+        create_test_repo(&dir);
         configure_git_cleanup_mode(&dir, "scissors");
         configure_git_comment_char(&dir, "-");
         let filename = "commit_message_file";
@@ -616,7 +707,7 @@ mod tests {
     fn test_file_option_without_file() {
         compile_bin();
         let dir = test_dir("commit_file_option_without_file");
-        create_test_repo(&dir, &[]);
+        create_test_repo(&dir);
         let filename = "commit_message_file";
 
         let mut cmd = assert_cmd::Command::cargo_bin("lintje").unwrap();
@@ -635,10 +726,8 @@ mod tests {
     fn test_branch_valid() {
         compile_bin();
         let dir = test_dir("branch_valid");
-        create_test_repo(
-            &dir,
-            &[("Test commit", "I am a test commit, short but valid.")],
-        );
+        create_test_repo(&dir);
+        create_commit_with_file(&dir, "Test commit", "I am a test commit.", "file");
         checkout_branch(&dir, "my-branch");
 
         let mut cmd = assert_cmd::Command::cargo_bin("lintje").unwrap();
@@ -652,11 +741,9 @@ mod tests {
     fn test_branch_invalid() {
         compile_bin();
         let dir = test_dir("branch_invalid");
-        create_test_repo(
-            &dir,
-            &[("Test commit", "I am a test commit, short but valid.")],
-        );
+        create_test_repo(&dir);
         checkout_branch(&dir, "fix-123");
+        create_commit_with_file(&dir, "Test commit", "I am a test commit.", "file");
 
         let mut cmd = assert_cmd::Command::cargo_bin("lintje").unwrap();
         let assert = cmd.current_dir(dir).assert().failure().code(1);
@@ -675,10 +762,8 @@ mod tests {
     fn test_no_branch_validation() {
         compile_bin();
         let dir = test_dir("branch_invalid_disabled");
-        create_test_repo(
-            &dir,
-            &[("Test commit", "I am a test commit, short but valid.")],
-        );
+        create_test_repo(&dir);
+        create_commit_with_file(&dir, "Test commit", "I am a test commit.", "file");
         checkout_branch(&dir, "fix-123");
 
         let mut cmd = assert_cmd::Command::cargo_bin("lintje").unwrap();
